@@ -21,7 +21,7 @@ use std::{
 
 use anyhow::{anyhow, bail};
 use colored::Colorize as _;
-use defmt_decoder::Locations;
+use defmt_decoder::{DecodeError, Frame, Locations, StreamDecoder};
 use probe_rs::{
     flashing::{self, Format},
     Core,
@@ -108,10 +108,14 @@ fn run_target_program(elf_path: &Path, chip_name: &str, opts: &cli::Opts) -> any
         .map(|canary| canary.touched(&mut core, elf))
         .transpose()?
         .unwrap_or(false);
-    let backtrace_settings = backtrace::Settings {
+
+    let panic_present = canary_touched || halted_due_to_signal;
+
+    let mut backtrace_settings = backtrace::Settings {
         current_dir,
-        max_backtrace_len: opts.max_backtrace_len,
-        force_backtrace: opts.force_backtrace || canary_touched || halted_due_to_signal,
+        backtrace_limit: opts.backtrace_limit,
+        backtrace: (&opts.backtrace).into(),
+        panic_present,
         shorten_paths: opts.shorten_paths,
     };
 
@@ -119,7 +123,7 @@ fn run_target_program(elf_path: &Path, chip_name: &str, opts: &cli::Opts) -> any
         &mut core,
         elf,
         &target_info.active_ram_region,
-        &backtrace_settings,
+        &mut backtrace_settings,
     )?;
 
     // if general outcome was OK but the user ctrl-c'ed, that overrides our outcome
@@ -216,12 +220,19 @@ fn extract_and_print_logs(
         bail!("\"defmt\" RTT channel is in use, but the firmware binary contains no defmt data");
     }
 
+    let mut decoder_and_encoding = if use_defmt {
+        elf.defmt_table
+            .as_ref()
+            .map(|table| (table.new_stream_decoder(), table.encoding()))
+    } else {
+        None
+    };
+
     print_separator();
 
     let stdout = io::stdout();
     let mut stdout = stdout.lock();
     let mut read_buf = [0; 1024];
-    let mut defmt_buffer = vec![];
     let mut was_halted = false;
     while !exit.load(Ordering::Relaxed) {
         if let Some(logging_channel) = &mut logging_channel {
@@ -234,16 +245,16 @@ fn extract_and_print_logs(
             };
 
             if num_bytes_read != 0 {
-                match &elf.defmt_table {
-                    Some(table) if use_defmt => {
-                        defmt_buffer.extend_from_slice(&read_buf[..num_bytes_read]);
+                match decoder_and_encoding.as_mut() {
+                    Some((stream_decoder, encoding)) => {
+                        stream_decoder.received(&read_buf[..num_bytes_read]);
 
                         decode_and_print_defmt_logs(
-                            &mut defmt_buffer,
-                            table,
+                            &mut **stream_decoder,
                             elf.defmt_locations.as_ref(),
                             current_dir,
-                            opts,
+                            opts.shorten_paths,
+                            encoding.can_recover(),
                         )?;
                     }
 
@@ -285,60 +296,65 @@ fn extract_and_print_logs(
 }
 
 fn decode_and_print_defmt_logs(
-    buffer: &mut Vec<u8>,
-    table: &defmt_decoder::Table,
+    stream_decoder: &mut dyn StreamDecoder,
     locations: Option<&Locations>,
     current_dir: &Path,
-    opts: &cli::Opts,
+    shorten_paths: bool,
+    encoding_can_recover: bool,
 ) -> anyhow::Result<()> {
     loop {
-        match table.decode(buffer) {
-            Ok((frame, consumed)) => {
-                // NOTE(`[]` indexing) all indices in `table` have already been verified to exist in
-                // the `locations` map
-                let (file, line, mod_path) = locations
-                    .map(|locations| &locations[&frame.index()])
-                    .map(|location| {
-                        let path = if let Ok(relpath) = location.file.strip_prefix(&current_dir) {
-                            relpath.display().to_string()
-                        } else {
-                            let dep_path = dep::Path::from_std_path(&location.file);
-
-                            if opts.shorten_paths {
-                                dep_path.format_short()
-                            } else {
-                                dep_path.format_highlight()
-                            }
-                        };
-
-                        (
-                            Some(path),
-                            Some(location.line as u32),
-                            Some(&*location.module),
-                        )
-                    })
-                    .unwrap_or((None, None, None));
-
-                // Forward the defmt frame to our logger.
-                defmt_decoder::log::log_defmt(&frame, file.as_deref(), line, mod_path);
-
-                let num_bytes = buffer.len();
-                buffer.rotate_left(consumed);
-                buffer.truncate(num_bytes - consumed);
-            }
-
-            Err(defmt_decoder::DecodeError::UnexpectedEof) => break,
-
-            Err(defmt_decoder::DecodeError::Malformed) => {
-                log::error!("failed to decode defmt data: {:x?}", buffer);
-                return Err(defmt_decoder::DecodeError::Malformed.into());
-            }
+        match stream_decoder.decode() {
+            Ok(frame) => forward_to_logger(&frame, locations, current_dir, shorten_paths),
+            Err(DecodeError::UnexpectedEof) => break,
+            Err(DecodeError::Malformed) => match encoding_can_recover {
+                // if recovery is impossible, abort
+                false => return Err(DecodeError::Malformed.into()),
+                // if recovery is possible, skip the current frame and continue with new data
+                true => continue,
+            },
         }
     }
 
     Ok(())
 }
 
+fn forward_to_logger(
+    frame: &Frame,
+    locations: Option<&Locations>,
+    current_dir: &Path,
+    shorten_paths: bool,
+) {
+    let (file, line, mod_path) = location_info(frame, locations, current_dir, shorten_paths);
+    defmt_decoder::log::log_defmt(frame, file.as_deref(), line, mod_path.as_deref());
+}
+
+fn location_info(
+    frame: &Frame,
+    locations: Option<&Locations>,
+    current_dir: &Path,
+    shorten_paths: bool,
+) -> (Option<String>, Option<u32>, Option<String>) {
+    locations
+        .map(|locations| &locations[&frame.index()])
+        .map(|location| {
+            let path = if let Ok(relpath) = location.file.strip_prefix(&current_dir) {
+                relpath.display().to_string()
+            } else {
+                let dep_path = dep::Path::from_std_path(&location.file);
+                match shorten_paths {
+                    true => dep_path.format_short(),
+                    false => dep_path.format_highlight(),
+                }
+            };
+            (
+                Some(path),
+                Some(location.line as u32),
+                Some(location.module.clone()),
+            )
+        })
+        .unwrap_or((None, None, None))
+}
+
 fn setup_logging_channel(
     rtt_buffer_address: u32,
     sess: Arc<Mutex<Session>>,
